#!/usr/bin/env python
# coding: utf-8

# In[11]:


# import and load data
#%matplotlib inline
import pandas as pd
import netCDF4
from netCDF4 import Dataset
import numpy as np
import gdal
from osgeo import gdal
#import matplotlib.pyplot as plt
#from mpl_toolkits.basemap import Basemap, cm
##from matplotlib.colors import BoundaryNorm, ColorConverter
from scipy.optimize import fsolve
import matplotlib.pyplot as plt
import scipy.stats as ss
import math
import pylab
from pylab import *
import os

#import datetime
from multiprocessing import Pool
from mpl_toolkits.basemap import Basemap, cm
#import copy
import gc
pyID = '02b'


# In[5]:


##load data npp
ras_npp=gdal.Open('Cropland2000_5m.tif')  

im_height = ras_npp.RasterYSize  
im_width = ras_npp.RasterXSize  

dtm_npp=ras_npp.ReadAsArray()  
print(im_height,im_width)
dtm_npp[dtm_npp < 0] = np.nan
dtm_npp[dtm_npp == 0] = np.nan
dtm_npp=dtm_npp*100
#plt.imshow(dtm_npp); plt.colorbar()
#dtm_npp=ras_npp.ReadAsArray(0,0,im_width,im_height)
print(dtm_npp)

##load SOC points
df1 = pd.read_excel('OtherSOC_total_220331.xlsx', 'All', index_col=None, na_values=['NA'])
print(df1.columns)


# In[3]:


fig = plt.figure(figsize=[14,8]) 
plt.subplots_adjust() 
#plt.subplots_adjust(left=0.05, right=0.95, hspace=0.1, wspace=-0.0) 
plt.rcParams.update({'font.size': 15})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
watercolor = 'lightskyblue'
landcolor = 'lightgrey'
lw=0.5

#------------main plot--------------

ax = fig.add_subplot(111,facecolor='white') 
#ax.set_title('a', loc=loc) 
m = Basemap(llcrnrlon=-180.,llcrnrlat=-60.,urcrnrlon=180.,urcrnrlat=90.) 
# m.drawmapboundary(fill_color=watercolor, zorder=0)
# m.fillcontinents(color=landcolor,lake_color=watercolor, zorder=0)
m.drawcoastlines(linewidth=0.5, color='gray')
#m.drawmeridians(np.arange(0, 360, 60),labels=[1,0,0,1]) 
#m.drawparallels(np.arange(-90, 90.001, 30), labels=[1,0,0,1]) 
grassmap = m.imshow(dtm_npp[:1800,:], origin='upper', cmap=plt.get_cmap('YlGn'))#PiYG
#data_legs=np.array('Li et al (194)','Morugancoronado et al (127)','Geisseler et al (38)','Sun et al (260)','This study (125)')

colors=['r','blue','orange','purple']
markers = np.array(['D','^','o','^']) # 
label=['This study (58 sites)','Sun et al. (157 sites)','Geisseler et al. (38 sites)','Zhou et al. (32 sites)']
#label=['Li et al (194)','Sun et al (127)','Geisseler et al (38)','Zhou et al (260)','This study (125)']
for index in range(4):
    lon=df1.loc[df1['index']==index]['LON']
    lat=df1.loc[df1['index']==index]['LAT']
    m.scatter(lon,lat, s=25, edgecolors=colors[index],color='white', alpha=0.5,marker=markers[index], label=label[index],linewidth=0.7, lw=0.8,zorder=2)
font = { 'size': 12}
ax.legend(loc=3, bbox_to_anchor=(-0.01,0.0),numpoints=1, frameon=False,prop=font)

cb = fig.colorbar(grassmap, orientation='horizontal',pad=0.02,shrink=0.5, aspect=25)
#cb.ax.set_xlabel('Percent area of cropland in each 10 x 10 km grid cell in 2000 (%)')
cb.ax.set_xlabel('Percent area of cropland (%)')
plt.show()
fig.savefig('fig%s_pointsmap.png'%(pyID),dpi=300)


# In[4]:


obs = pd.read_excel('Table_SI_210501_testbd_update220409.xlsx', 'TableS1', index_col=None, na_values=['NA']) #不分
obs=obs[~obs.Rate_BC.isin([0])]
obs = obs.reset_index(drop=True)
df1 = obs.dropna(subset=['Rate_BC'])
df1 = df1.dropna(subset=['Age_BC'])
print ('min and maxmium of rate_bc:',np.nanmin(df1['Rate_BC']),np.nanmax(df1['Rate_BC']))
print ('averaged and median of rate_bc:',np.nanmean(df1['Rate_BC']),np.nanmedian(df1['Rate_BC']))
print ('averaged and median of age_bc:',np.nanmean(df1['Age_BC']),np.nanmedian(df1['Age_BC']))
#--------------------------------------
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.pyplot import MultipleLocator

fig=plt.figure(1,figsize=(8,4))
plt.subplots_adjust(left=0.1, bottom=0.2, right=0.94, top=0.93, wspace=0.25, hspace=0.2)  
plt.rcParams.update({'font.size': 12})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
ax1 = plt.subplot(1,2,1)

kwargs = dict(histtype='stepfilled', alpha=0.8,density=True, bins=50)
ax1.hist(df1['Rate_BC'], **kwargs,color='deepskyblue',label='Rate_BC')
ax1.axvline(np.nanmedian(df1['Rate_BC']),ls='--', lw=1., c='r') 
plt.xlim(-5,100)
ax1.set_xlabel('Rate_BC (t ha$\mathdefault{^{-1}}$)')
ax1.set_ylabel('Frequency')
plt.title('(a)',weight='bold',loc='left')


ax2 = plt.subplot(1,2,2)
ax2.hist(df1['Age_BC'], **kwargs,color='deepskyblue',label='Rate_BC')
ax2.axvline(np.nanmedian(df1['Age_BC']),ls='--', lw=1., c='r') 
ax2.set_xlabel('Age_BC (year$\mathdefault{^{ }}$)')
#ax2.set_ylabel('Frequency')
plt.title('(b)',weight='bold',loc='left')
fig.savefig('fig%s_frequency.png'%(pyID),dpi=300)


# In[16]:


obs = pd.read_excel('Table_SI_210501_testbd_update220409.xlsx', 'TableS1', index_col=None, na_values=['NA']) 
obs=obs[obs.Rate_BC.isin([0])]
obs = obs.reset_index(drop=True)
df1 = obs.dropna(subset=['Rate_BC'])
df1 = df1.dropna(subset=['Age_BC'])
print(len(df1['Age_BC'][df1['Age_BC']<3])/len(df1['Age_BC']))


# In[3]:


fig=plt.figure(1,figsize=(12,4))
plt.subplots_adjust(left=0.1, bottom=0.2, right=0.94, top=0.9, wspace=0.25, hspace=0.2)  
plt.rcParams.update({'font.size': 12})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})

ax1 = plt.subplot(1,3,1)
theta1=np.arange(0,0.58,0.01)
phi=0.58
nx=len(theta1)
fm2=np.zeros(nx)
for i in range(nx):
    fm2[i]=1/(1+30*math.exp(-8.5*theta1[i]/phi))
print(fm2)
ax1.plot(theta1/phi,fm2,ls='-', lw=2, c='dodgerblue',label='Century')
#ax.plot(theta1,fm2,ls='-', lw=0.8, c='b')
ax1.set_xlabel('Moisture index(w)')
ax1.set_ylabel('f (w)')
plt.title('(a) Century',weight='bold',loc='left')


ax2 = plt.subplot(1,3,2)
moist_in=np.arange(0,1,0.001)
nx=len(moist_in)
fm11=np.zeros(nx)
for i in range(nx):
    moist_coeff1=1.1
    moist_coeff2=2.4
    moist_coeff3=0.29
    moistcont_min=0.25
    #moistfunc_result=-moist_coeff1*moist_in[i]*moist_in[i] + moist_coeff2*moist_in[i] - moist_coeff3
    moistfunc_result=moist_coeff1*moist_in[i]*moist_in[i] + moist_coeff2*moist_in[i] + moist_coeff3
    control_moist=max(moistcont_min, min(1,moistfunc_result))  
    fm11[i]=control_moist
ax2.plot(moist_in,fm11,ls='-', lw=2, c='dodgerblue',label='ORCHIDEE')
ax2.set_xlabel('Water content (θ) (m$\mathdefault{^{3}}$/m$\mathdefault{^{3}}$)')
ax2.set_ylabel('f (θ)')
plt.title('(b) ORCHIDEE',weight='bold',loc='left')


ax3 = plt.subplot(1,3,3)
#---------------------Yan et al ---------------------------
# global varivables
#phi=0.58  ## soil porosity,
c=0 # ？g/g
ms=1.5 # cementation exponent for solute diffusion
ns=2   # saturation exponent for solute diffusion
mg=1.5 # cementation exponent for gas diffusion
ng=2.5 # saturation exponent for gas diffusion
V_DO=2.45 # g/g, stoichiometric coefficient of DO
K_theta=0.1 # m3/m3, moisture constant
alpha=2e-8 #1/s, mass transfer coefficient
rho_s=2.65e3 #kg/m3, density of soil mineral

H=1 # m, that is 20cm

#C_soc=0.02 #g/g
m_soc=3
D_GO=2.1e-5 # m2/s, diffusion coefficient of gaseous O2

#------------------def function -----------------------
def func_moist(sm,rho_b,clay,H,m_soc):
    # Calculate phi (φ)
    phi=1-rho_b/rho_s  #rho_b: soil bulk density,kg/m3, input1
    RSM=sm/phi

    # Calculate a
    def func_a(clay):
        if clay <0.016:
            a=0
        elif clay > 0.37:
            a=1
        else:
            a=2.8*clay-0.046
        return a
    a=func_a(clay)
    # Calculate b
    #b=0.75 # for heterogenous soils
    b=1.7 # for homogeneous soils

    # Calculate m_soc,unite:kg/m2
    #m_soc=rho_s*(1-phi)*H*C_soc
    C_soc=m_soc/(rho_s*(1-phi)*H)
    # Calculate theta_op 
    def symfun(x):   # phi,a,b,C_soc,m_soc
        f=V_DO*x/(K_theta+x)*alpha*m_soc*phi**(a*(ms-ns))*x**(a*ns)-(1.72*C_soc+0.065)*phi**(mg-ng)*(phi-x)**b*D_GO
        return f
    def bisection(a,b,eps):
        if symfun(a)*symfun(b)>0:
            print('Wrong choice bro')
        else:
            p=(a+b)/2
            err=abs(symfun(p))
            while err>eps:
                if symfun(a)*symfun(p) <0:
                    b=p
                else:
                    a=p
                p=(a+b)/2
                err=abs(symfun(p))
        return p
    theta_op=bisection(0,phi,1e-9)  #
    #print(theta_op)

    # Calculate fm
    def func_fm(RSM,phi,a,b,theta_op):
        if RSM < theta_op/phi :
            fth=(K_theta+theta_op)/theta_op**(1+a*ns)*(RSM*phi)**(1+a*ns)/(K_theta+RSM*phi)
        else:
            fth=(phi-RSM*phi)**b/(phi-theta_op)**b
        return fth

    fm=func_fm(RSM,phi,a,b,theta_op)
    return fm
#moist_control=func_moist(sm,rho_b,clay,H,C_soc)
phi=0.58
moist_in=np.arange(0,0.58,0.01)
out1=np.zeros(len(moist_in))
k=-1
for i in moist_in:
    k=k+1
    litter_moist=func_moist(i,1113,0.2,1,3)
    out1[k]=litter_moist
#print(out1)
#ax.plot(np.linspace(0, 100, 102),ls='-', lw=0.8, c='r')
ax3.plot(moist_in/phi, out1,ls='-', lw=2, c='dodgerblue',label='Yan et al')
ax3.set_xlabel('Relative water content (θ/φ)')
ax3.set_ylabel('f (θ/φ)')
#ax3.legend(fontsize=12,frameon=False)
plt.title('(c) Yan et al., (2018)',weight='bold',loc='left')

fig.savefig('fig%s_sm_func.png'%(pyID),dpi=300)


# In[12]:


## -------------------------*line fit*-------------------------
from scipy import optimize
import numpy as np
from scipy import stats
import statsmodels.api as sm
import math

def f_1(xx,A,B):
    return A*xx + B


## 求R2
def get_r2_numpy(x, y):
    slope, intercept = np.polyfit(x, y, 1)
    r_squared = 1 - (sum((y - (slope * x + intercept))**2) / ((len(y) - 1) * np.var(y, ddof=1)))
    return r_squared


# In[264]:


a1=np.load('MIMICS-TSMb-biochar1_ACsoc_ge3year_gkg.npy')
a3=np.load('/MIMICS-TSMb-biochar3_1_ACsoc_ge3year_gkg.npy')
a4=np.load('MIMICS-TSMb-biochar4_1_ACsoc_ge3year_gkg.npy')

b1=np.load('MIMICS-TSMb-biochar1_ACsoc_ge4year_gkg.npy')
b3=np.load('MIMICS-TSMb-biochar3_1_ACsoc_ge4year_gkg.npy')
b4=np.load('MIMICS-TSMb-biochar4_1_ACsoc_ge4year_gkg.npy')

c1=np.load('MIMICS-TSMb-biochar1_ACsoc_ge5year_gkg.npy')
c3=np.load('MIMICS-TSMb-biochar3_1_ACsoc_ge5year_gkg.npy')
c4=np.load('MIMICS-TSMb-biochar4_1_ACsoc_ge5year_gkg.npy')

d1=np.load('MIMICS-TSMb-biochar1_ACsoc_ge6year_gkg.npy')
d3=np.load('MIMICS-TSMb-biochar3_1_ACsoc_ge6year_gkg.npy')
d4=np.load('MIMICS-TSMb-biochar4_1_ACsoc_ge6year_gkg.npy')

y=[a1[1],a3[1],a4[1],b1[1],b3[1],b4[1],c1[1],c3[1],c4[1],d1[1],d3[1],d4[1]]
x=[a1[0],a3[0],a4[0],b1[0],b3[0],b4[0],c1[0],c3[0],c4[0],d1[0],d3[0],d4[0]]


title=['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)','(j)','(k)','(l)']
num_params==[0,1,2,0,1,2,0,1,2,0,1,2]
#---plot-------
fig=plt.figure(figsize=(10,12))
plt.subplots_adjust(left=0.1, bottom=0.1, right=0.95, top=0.95, wspace=0., hspace=0.)  
plt.rcParams.update({'font.size': 15})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
for i in range(12):
    
    ax1 = fig.add_subplot(4,3,i+1)
    ax1.scatter(x[i],y[i], color='dodgerblue', s=16, edgecolors='k',linewidth=0.2)
    ax1.plot(np.arange(-5,60,10),np.arange(-5,60,10),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left',y=0.9)    
    if i==3:
        ax1.set_ylabel('Simulated ΔSOC (g kg$\mathdefault{^{-1}}$)',y=0.)
    if i==10:
        ax1.set_xlabel('Observed ΔSOC (g kg$\mathdefault{^{-1}}$)')
        
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    R2=get_r2_numpy(x[i], y[i])
    ndf1=len(x[i])  #a1.shape[1]
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105        
#----------------------
#----------------------
    # aic
    def calculate_aic(n, rmse, num_params):
        aic = n * np.log(rmse) + 2 * num_params
        return aic
    #num_params=7#param_def.shape[1]
    aic= calculate_aic(ndf1, RMSE**2, num_params[i])
    print('AIC: %.3f' % aic)
#------------------------
    if i<3:  
        plt.xlim(-5,50)
        plt.ylim(-5,50)
        fig.text(0.33+i/3.5,0.90,"1:1",size=12)

        fig.text(0.13+i/3.5,0.92,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f g kg$\mathdefault{^{-1}}$'%(R2,RMSE),size=12)
        fig.text(0.13+i/3.5,0.90,'AIC =%.2f'%(aic),size=12)
        fig.text(0.13+i/3.5,0.88,'Slope =%.2f'%(A1),size=12)
        fig.text(0.13+i/3.5,0.86,'N =%.0f'%(a1.shape[1]),size=12)
        xx1=np.arange(-3,45,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i>2 and i<6:
        #ax1.set_xticks(np.arange(0,60,10))
        plt.xlim(-5,50)
        plt.ylim(-5,50)
        fig.text(0.61+(i-4)/3.5,0.68,"1:1",size=12)
        
        fig.text(0.42+(i-4)/3.5,0.70,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f g kg$\mathdefault{^{-1}}$'%(R2,RMSE),size=12)
        fig.text(0.42+(i-4)/3.5,0.68,'AIC =%.2f'%(aic),size=12)
        fig.text(0.42+(i-4)/3.5,0.66,'Slope =%.2f'%(A1),size=12)
        fig.text(0.42+(i-4)/3.5,0.64,'N =%.0f'%(b1.shape[1]),size=12)
        xx1=np.arange(-3,45,5)

        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i>5 and i<9:
        plt.xlim(-5,50)
        plt.ylim(-5,50)  
        fig.text(0.9+(i-8)/3.5,0.47,"1:1",size=12)
        
        fig.text(0.71+(i-8)/3.5,0.49,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f g kg$\mathdefault{^{-1}}$'%(R2,RMSE),size=12)
        fig.text(0.71+(i-8)/3.5,0.47,'AIC =%.2f'%(aic),size=12)
        fig.text(0.71+(i-8)/3.5,0.45,'Slope =%.2f'%(A1),size=12)
        fig.text(0.71+(i-8)/3.5,0.43,'N =%.0f'%(c1.shape[1]),size=12)
        
        xx1=np.arange(-3,45,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)     
    if i>8:
        plt.xlim(-5,50)
        plt.ylim(-5,50) 
        fig.text(1.19+(i-12)/3.5,0.26,"1:1",size=12)
        
        fig.text(1+(i-12)/3.5,0.28,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f g kg$\mathdefault{^{-1}}$'%(R2,RMSE),size=12)
        fig.text(1+(i-12)/3.5,0.26,'AIC =%.2f'%(aic),size=12)
        fig.text(1+(i-12)/3.5,0.24,'Slope =%.2f'%(A1),size=12)
        fig.text(1+(i-12)/3.5,0.22,'N =%.0f'%(d1.shape[1]),size=12)
        
        xx1=np.arange(-3,45,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)  
#----------------------
    if i!=0 and i!=3 and i!=6 and i!=9:   
        ax1.set_yticklabels([])
    if i<9:
        ax1.set_xticklabels([])
        
fig.savefig('mimics_output/fig%s_shortSOC_ge.png'%(pyID),dpi=300)      


# In[15]:



c2=np.load('MIMICS-T-sorption-century_merge.npy')
c3=np.load('MIMICS-T-sorption-orchidee_merge.npy')
c4=np.load('H/MIMICS-T-sorption-yan_merge.npy')

y=[c2[1],c3[1],c4[1]]
x=[c2[0],c3[0],c4[0]]


title=['(a) MIMICS-$\mathregular{TSM_a}$','(b) MIMICS-$\mathregular{TSM_b}$','(c) MIMICS-$\mathregular{TSM_c}$']
num_params=[7,7,7]
#---plot-------
fig=plt.figure(figsize=(10,4))
plt.subplots_adjust(left=0.1, bottom=0.2, right=0.95, top=0.9, wspace=0., hspace=0.)  
plt.rcParams.update({'font.size': 13})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
for i in range(3):
    
    ax1 = fig.add_subplot(1,3,i+1)
    ax1.scatter(x[i],y[i], color='dodgerblue', s=16, edgecolors='k',linewidth=0.2)
    ax1.plot(np.arange(0,60,5),np.arange(0,60,5),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left')    
    if i==0:
        ax1.set_ylabel('Simulated SOC concentration (g kg$\mathdefault{^{-1}}$)')
    if i==1:
        ax1.set_xlabel('Observed SOC concentration (g kg$\mathdefault{^{-1}}$)',x=0.5)
        
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    print(A1)
    R2=get_r2_numpy(x[i], y[i])
    ndf1=b1.shape[1]
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105        
#----------------------
#----------------------
    # aic
    def calculate_aic(n, rmse, num_params):
        aic = n * np.log(rmse) + 2 * num_params
        return aic
    #num_params=7#param_def.shape[1]
    aic= calculate_aic(ndf1, RMSE**2, num_params[i])
    print('AIC: %.3f' % aic)
#------------------------
    ax1.set_yticks(np.arange(0,50,10))
    ax1.set_xticks(np.arange(0,50,10))    
    if i<3:  
        plt.xlim(0,50)
        plt.ylim(0,50)
        fig.text(0.33+i/3.5,0.75,"1:1",size=12)

        fig.text(0.13+i/3.5,0.8,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f'%(R2,RMSE),size=12)
        fig.text(0.13+i/3.5,0.72,'N =%.0f'%(b1.shape[1]),size=12)
        fig.text(0.13+i/3.5,0.76,'AIC =%.1f'%(aic),size=12)
        xx1=np.arange(1,46,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
#----------------------
    if i!=0 :   
        ax1.set_yticklabels([])
    #if i<3:
        #ax1.set_xticklabels([])
fig.savefig('mimics_output/fig%s_sensi.png'%(pyID),dpi=300)  


# In[3]:


from mpl_toolkits.axes_grid1 import host_subplot
from mpl_toolkits import axisartist
import matplotlib.pyplot as plt
fig=plt.figure(figsize=(10,5.))
host = host_subplot(111, axes_class=axisartist.Axes)
plt.subplots_adjust(left=0.1, bottom=0.1, right=0.85, top=0.9, wspace=0., hspace=0.)  
plt.rcParams.update({'font.size': 13})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
#plt.subplots_adjust(right=0.75)
par1 = host.twinx()
par2 = host.twinx()

par2.axis["right"] = par2.new_fixed_axis(loc="right", offset=(60, 0))

par1.axis["right"].toggle(all=True)
par2.axis["right"].toggle(all=True)

p1 = host.bar([0,1,2,3,4,5], [0.38, 0.47, 0.51,0.50,0.52,0.50], label="R2",width=0.2,color='skyblue')
p2 = par1.bar([0.22,1.22,2.22,3.22,4.22,5.22], [4.96, 4.61, 4.45,4.53,4.41,4.48], label="RMSE",width=0.2,color='#bbded6')
p3= par2.bar([0.44,1.44,2.44,3.44,4.44,5.44], [191.8, 185.3, 187.2,189.2,186,188], label="AIC",width=0.2,color='#ffb6b9')
x5=np.array([0,1,2,3,4,5])
host.set_xticks(x5+0.22)
host.set_xticklabels([])
host.set_xticklabels(['MIMICS-def','MIMICS-T','MIMICS-TS','MIMICS-$\mathregular{TSM_a}$','MIMICS-$\mathregular{TSM_b}$',
                      'MIMICS-$\mathregular{TSM_c}$'],rotation='45',horizontalalignment='right',size=14)
#host.set_xticklabels([])
#host.set_xlim(0, 2)
host.set_ylim(0, 0.6)
par1.set_ylim(0, 8)
par2.set_ylim(100, 220)

#host.set_xlabel("MIMICS Models")
host.set_ylabel("R$\mathdefault{^{2}}$")
par1.set_ylabel("RMSE (g kg$\mathdefault{^{-1}}$)")
par2.set_ylabel("AIC")

plt.bar_label(p1, label_type='edge',size=10)
par1.bar_label(p2, label_type='edge',size=10)
par2.bar_label(p3, label_type='edge',size=10)

host.legend(frameon=False,ncol=3,loc='upper center')

plt.show()
fig.savefig('mimics_output/fig%s_aic.png'%(pyID),dpi=300)  


# In[255]:



a1=np.load('mimics_input/MIMICS-def_sun.npy')
a2=np.load('mimics_input/MIMICS-T_sun.npy')
a3=np.load('mimics_input/MIMICS-TS_sun.npy')
a4=np.load('mimics_input/MIMICS-TSMb_sun.npy')

b1=np.load('mimics_input/MIMICS-def_gei.npy')
b2=np.load('mimics_input/MIMICS-T_gei.npy')
b3=np.load('mimics_input/MIMICS-TS_gei.npy')
b4=np.load('mimics_input/MIMICS-TSMb_gei.npy')

c1=np.load('mimics_input/MIMICS-def_zho.npy')
c2=np.load('mimics_input/MIMICS-T_zho.npy')
c3=np.load('mimics_input/MIMICS-TS_zho.npy')
c4=np.load('mimics_input/MIMICS-TSMb_zho.npy')


y=[a1[1],a2[1],a3[1],a4[1],b1[1],b2[1],b3[1],b4[1],c1[1],c2[1],c3[1],c4[1]]
x=[a1[0],a2[0],a3[0],a4[0],b1[0],b2[0],b3[0],b4[0],c1[0],c2[0],c3[0],c4[0]]


title=['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)','(j)','(k)','(l)','(m)','(n)','(o)','(p)']
num_params=[3,4,7,7,3,4,7,7,3,4,7,7]
#---plot-------
fig=plt.figure(figsize=(16,12),facecolor='white')
plt.subplots_adjust(left=0.1, bottom=0.1, right=0.95, top=0.95, wspace=0., hspace=0.)  
plt.rcParams.update({'font.size': 15})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
for i in range(12):
    
    ax1 = fig.add_subplot(3,4,i+1)
    ax1.scatter(x[i],y[i], color='dodgerblue', s=16, edgecolors='k',linewidth=0.2)
    ax1.plot(np.arange(0,70,10),np.arange(0,70,10),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left',y=0.9)    
    if i==4:
        ax1.set_ylabel('Simulated SOC concentrations (g kg$\mathdefault{^{-1}}$)',y=0.5)
    if i==9:
        ax1.set_xlabel('Observed SOC concentrations (g kg$\mathdefault{^{-1}}$)',x=1)
        
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    R2=get_r2_numpy(x[i], y[i])
    ndf1=len(x[i])  #a1.shape[1]
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)        
#----------------------
#----------------------
    # aic
    def calculate_aic(n, rmse, num_params):
        aic = n * np.log(rmse) + 2 * num_params
        return aic
    #num_params=7#param_def.shape[1]
    aic= calculate_aic(ndf1, RMSE**2, num_params[i])
    print('AIC: %.3f' % aic)
#------------------------
    if i<4:  
        plt.xlim(0,53)
        plt.ylim(0,53)
        fig.text(0.28+i/4.8,0.88,"1:1",size=15)

        fig.text(0.13+i/4.5,0.90,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f'%(R2,RMSE),size=13)
        fig.text(0.13+i/4.5,0.88,'AIC =%.1f'%(aic),size=13)
        fig.text(0.13+i/4.5,0.86,'N =%.0f'%(a1.shape[1]),size=13)
        xx1=np.arange(2,50,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i>3 and i<8:
        #ax1.set_xticks(np.arange(0,60,10))
        plt.xlim(0,53)
        plt.ylim(0,53)
        fig.text(0.28+(i-4)/4.8,0.60,"1:1",size=15)
        
        fig.text(0.13+(i-4)/4.5,0.62,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f'%(R2,RMSE),size=13)
        fig.text(0.13+(i-4)/4.5,0.60,'AIC =%.1f'%(aic),size=13)
        fig.text(0.13+(i-4)/4.5,0.58,'N =%.0f'%(b1.shape[1]),size=13)
        xx1=np.arange(2,50,5)

        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i>7 and i<12:
        plt.xlim(0,53)
        plt.ylim(0,53)  
        fig.text(0.28+(i-8)/4.8,0.32,"1:1",size=15)
        
        fig.text(0.13+(i-8)/4.5,0.34,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f'%(R2,RMSE),size=13)
        fig.text(0.13+(i-8)/4.5,0.32,'AIC =%.0f'%(aic),size=13)
        fig.text(0.13+(i-8)/4.5,0.3,'N =%.0f'%(c1.shape[1]),size=13)
        xx1=np.arange(2,50,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)     
 
#----------------------
    if i!=0 and i!=4 and i!=8 and i!=12:   
        ax1.set_yticklabels([])
    if i<8:
        ax1.set_xticklabels([])
        
fig.savefig('fig%s_shortSOC_ge.png'%(pyID),dpi=300)      


# In[5]:


a1=np.load('mimics_input/MIMICS-TSMb-maize.npy')
a2=np.load('mimics_input/MIMICS-TSMb-rice.npy')
a3=np.load('mimics_input/MIMICS-TSMb-wheat.npy')


y=[a1[1],a2[1],a3[1]]
x=[a1[0],a2[0],a3[0]]

title=['(a)','(b)','(c)']
#---plot-------
fig=plt.figure(figsize=(10,4))
plt.subplots_adjust(left=0.1, bottom=0.15, right=0.95, top=0.9, wspace=0., hspace=0.)  
plt.rcParams.update({'font.size': 13})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
for i in range(3):
    
    ax1 = fig.add_subplot(1,3,i+1)
    ax1.scatter(x[i],y[i], color='dodgerblue', s=16, edgecolors='k',linewidth=0.2)
    ax1.plot(np.arange(0,60,5),np.arange(0,60,5),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left')    
    plt.xlim(0,50)
    plt.ylim(0,50)
    ax1.set_xticks(np.arange(0,50,10))
        
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    R2=get_r2_numpy(x[i], y[i])
    ndf1=len(y[i])
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105        
#----------------------
    if i==0:
        ax1.set_xlabel('Observed SOC concentration (g kg$\mathdefault{^{-1}}$)',x=1.5)
        ax1.set_ylabel('Simulated SOC concentration (g kg$\mathdefault{^{-1}}$)')
    if i!=0:
        ax1.set_yticklabels([])
    fig.text(0.33+i/3.5,0.7,"1:1",size=12)
    fig.text(0.12+i/3.5,0.75,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f'%(R2,RMSE),size=12)
    fig.text(0.12+i/3.5,0.7,'N =%.0f'%(len(y[i])),size=12)
    xx1=np.arange(1,46,5)
    yy1=A1 *xx1 +B1
    ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
fig.savefig('fig%s_sensi.png'%(pyID),dpi=300)  


# In[55]:


obs = pd.read_excel('Table_SI_210501_cess_allupdate.xlsx', 'TableS1', index_col=None, na_values=['NA']) #不分
obs = obs.reset_index(drop=True)
df1 = obs.dropna(subset=['Rate_BC'])
df1 = df1.dropna(subset=['Age_BC'])
print(df1.columns)
df_maize=df1[df1.CropType.isin(['maize'])]
df_rice=df1[df1.CropType.isin(['rice'])]
df_wheat=df1[df1.CropType.isin(['wheat'])]
print('maize:',len(df_maize),'\n','rice:',len(df_rice),'\n','wheat:',len(df_wheat))


x=[df_maize['NPP_MODIS'],df_maize['MAT'],df_maize['Clay']*100,
   df_rice['NPP_MODIS'],df_rice['MAT'],df_rice['Clay']*100,
   df_wheat['NPP_MODIS'],df_wheat['MAT'],df_wheat['Clay']*100]
    
y=[df_maize['SOC'],df_maize['SOC'],df_maize['SOC'],
   df_rice['SOC'],df_rice['SOC'],df_rice['SOC'],
   df_wheat['SOC'],df_wheat['SOC'],df_wheat['SOC']]

title=['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)']
#---plot-------
fig=plt.figure(figsize=(10,10),facecolor='white')
plt.subplots_adjust(left=0.1, bottom=0.1, right=0.95, top=0.95, wspace=0., hspace=0.25)  
plt.rcParams.update({'font.size': 13})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})

for i in range(9):
    
    ax1 = fig.add_subplot(3,3,i+1)
    #ax1.scatter(x[i],y[i], color='skyblue', s=16, edgecolors='k',linewidth=0.2)
    ax1.errorbar(x[i],y[i],fmt='o', mec='black', mfc='skyblue',ms=6,ecolor='skyblue',elinewidth=0.6,capthick=0.6,capsize=2)
    #ax1.plot(np.arange(-5,65,5),np.arange(-5,65,5),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left')    
    if i==3 :
        ax1.set_ylabel('SOC concentration (g kg$\mathdefault{^{-1}}$)')
    if i==6:
        ax1.set_xlabel('NPP (g C m$\mathdefault{^{-2}}$ yr$\mathdefault{^{-1}}$)')
    if i==7:
        ax1.set_xlabel('MAT (°C)')
    if i==8:
        ax1.set_xlabel('Clay (%)')
        
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    R2=get_r2_numpy(x[i], y[i])
    ndf1=len(x[i])
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105        
#-------------------
    if i<3:  
        fig.text(0.2+i/3.5,0.92,'R$\mathdefault{^{2}}$=%.2f'%(R2),size=12)
        fig.text(0.2+i/3.5,0.90,'N =%.0f'%(len(x[i])),size=12)
    if i>2 and i<6:
        fig.text(0.2+(i-3)/3.5,0.60,'R$\mathdefault{^{2}}$=%.2f'%(R2),size=12)
        fig.text(0.2+(i-3)/3.5,0.58,'N =%.0f'%(len(x[i])),size=12)
    if i>5:
        fig.text(0.2+(i-6)/3.5,0.30,'R$\mathdefault{^{2}}$=%.2f'%(R2),size=12)
        fig.text(0.2+(i-6)/3.5,0.28,'N =%.0f'%(len(x[i])),size=12)
#---------------------
    if i!=0 and i!=3 and i!=6:
        ax1.set_yticklabels([])
    if i==0:
        xx1=np.arange(150,900,20)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i==1:
        xx1=np.arange(5,25,2)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i==2:
        xx1=np.arange(10,50,2)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.) 
    if i==3:
        xx1=np.arange(450,650,20)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i==4:
        xx1=np.arange(12,25,2)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)        
    if i==5:
        xx1=np.arange(10,45,2)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)     
        
    if i==6:
        xx1=np.arange(100,500,20)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)         
    if i==7:
        xx1=np.arange(5,25,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)  
    if i==8:
        xx1=np.arange(10,40,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)  
fig.savefig('mimics_output/fig%s_croptype.png'%(pyID),dpi=300)  


# In[10]:


a1=np.load('MIMICS-TSMb_smtest.npy')
a2=np.load('MIMICS-TSMb_nppopt.npy')
y=[a1[1],a2[1]]
x=[a1[0],a2[0]]

title=['(a) ','(b) ']
#---plot-------
fig=plt.figure(figsize=(8,4.5))
plt.subplots_adjust(left=0.1, bottom=0.15, right=0.95, top=0.9, wspace=0., hspace=0.)  
plt.rcParams.update({'font.size': 13})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
num_params=np.array([7,8])
for i in range(2):
    
    ax1 = fig.add_subplot(1,2,i+1)
    ax1.scatter(x[i],y[i], color='dodgerblue', s=16, edgecolors='k',linewidth=0.2)
    ax1.plot(np.arange(0,60,5),np.arange(0,60,5),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left')    
    plt.xlim(0,50)
    plt.ylim(0,50)
    ax1.set_xticks(np.arange(0,50,10))
        
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    R2=get_r2_numpy(x[i], y[i])
    ndf1=len(y[i])
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105        
#----------------------
    # 定义aic计算函数
    def calculate_aic(n, rmse, num_params):
        aic = n * np.log(rmse) + 2 * num_params
        return aic
    #num_params=7#param_def.shape[1]
    aic= calculate_aic(ndf1, RMSE**2, num_params[i])
    #print('AIC: %.3f' % aic)
#----------------------
    if i==0:
        ax1.set_xlabel('Observed SOC concentration (g kg$\mathdefault{^{-1}}$)',x=1)
        ax1.set_ylabel('Simulated SOC concentration (g kg$\mathdefault{^{-1}}$)')
    if i!=0:
        ax1.set_yticklabels([])
    fig.text(0.45+i/2.3,0.7,"1:1",size=12)
    fig.text(0.12+i/2.3,0.75,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f g kg$\mathdefault{^{-1}}$'%(R2,RMSE),size=12)
    fig.text(0.12+i/2.3,0.7,'AIC =%.0f'%(aic),size=12)    
    
    fig.text(0.12+i/2.3,0.65,'N =%.0f'%(len(y[i])),size=12)
    xx1=np.arange(1,46,5)
    yy1=A1 *xx1 +B1
    ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
fig.savefig('mimics_output/fig%s_test.png'%(pyID),dpi=300)  


# In[104]:


a1=np.load('MIMICS-Def-merge.npy')
a2=np.load('MIMICS-T-merge.npy')
a3=np.load('MIMICS-T-sorption-merge.npy')
a4=np.load('MIMICS-T-ORCHIDEE-merge.npy')
a5=np.load('MIMICS-T-sorption-orchidee-merge.npy')

y=[a1[1],a2[1],a3[1],a5[1]]
x=[a1[0],a2[0],a3[0],a5[0]]


title=['(a) MIMICS-def','(b) MIMICS-T','(c) MIMICS-TS','(d) MIMICS-$\mathregular{TSM_b}$']
#num_params=np.array([7,9,10,10,10])
num_params=np.array([3,4,7,7])
#---plot-------
fig=plt.figure(figsize=(12,8.2),facecolor='white')
plt.subplots_adjust(left=0.1, bottom=0.1, right=0.95, top=0.95, wspace=0.1, hspace=0.15)  
plt.rcParams.update({'font.size': 15})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
for i in range(4):
    
    ax1 = fig.add_subplot(2,3,i+1)
    ax1.scatter(x[i],y[i], color='#2BBBD8', s=25, edgecolors='k',linewidth=0.5)
    ax1.plot(np.arange(0,60,5),np.arange(0,60,5),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left')    
    plt.xlim(0,40)
    plt.ylim(0,40)
    ax1.set_xticks(np.arange(0,50,10))
    ax1.set_yticks(np.arange(0,50,10))
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    print(A1)
    R2=get_r2_numpy(x[i], y[i])
    ndf1=len(y[i])
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105        
#----------------------
    # aic
    def calculate_aic(n, rmse, num_params):
        aic = n * np.log(rmse) + 2 * num_params
        return aic
    #num_params=10#param_def.shape[1]
    aic= calculate_aic(ndf1, RMSE**2, num_params[i])
    #print('AIC: %.3f' % aic)
#------------------------
    if i==3:
        ax1.set_xlabel('Observed SOC concentration (g kg$\mathdefault{^{-1}}$)')
    if i==0:   
        ax1.set_ylabel('Simulated SOC concentration (g kg$\mathdefault{^{-1}}$)',y=0)
    if i!=0 and i!=3:
        ax1.set_yticklabels([])
    if i!=3:
        ax1.set_xticklabels([])
    
    if i<3: 
        fig.text(0.32+i/3.4,0.85,"1:1",size=12)
        #fig.text(0.12+i/3.4,0.88,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f g kg$\mathdefault{^{-1}}$'%(R2,RMSE),size=12)
        fig.text(0.12+i/3.4,0.85,'R$\mathdefault{^{2}}$=%.2f'%(R2),size=13)
        fig.text(0.12+i/3.4,0.82,'N =%.0f'%(len(y[i])),size=13)
        #fig.text(0.12+i/3.4,0.82,'AIC =%.1f'%(aic),size=12)
        fig.text(0.12+i/3.4,0.89,'Y=%.2fX+%.2f'%(A1,B1),size=13)
        xx1=np.arange(1,35,2)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    else:
        fig.text(-0.58+i/3.34,0.4,"1:1",size=12)
        #fig.text(-0.78+i/3.33,0.43,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f g kg$\mathdefault{^{-1}}$'%(R2,RMSE),size=12)
        fig.text(-0.78+i/3.33,0.4,'R$\mathdefault{^{2}}$=%.2f'%(R2),size=13)
        fig.text(-0.78+i/3.33,0.37,'N =%.0f'%(len(y[i])),size=13)
        #fig.text(-0.78+i/3.33,0.37,'AIC =%.1f'%(aic),size=12)
        fig.text(-0.78+i/3.33,0.44,'Y=%.2fX+%.2f'%(A1,B1),size=13)        
        xx1=np.arange(1,35,2)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)   
#####################

ix, iy, w0, h0 = 0.44, 0.15, 0.21, 0.344
ax5 = fig.add_axes([ix, iy, w0, h0])#, facecolor='none'
#ax5.set_title('(e)', weight='bold', loc='left')
#fig.text(0.545,0.493,'(e)',weight='bold',size=15)
ax5.set_title('(e)', weight='bold', loc='left')    

x5=np.arange(4)
rmse5=np.array([0.38,0.47,0.51,0.52])
p1=ax5.bar(x5,rmse5,color='skyblue',width=0.32) #'#8ac6d1'
ax5.set_ylabel('R$\mathdefault{^{2}}$',color='dodgerblue')
ax5.tick_params(axis='y',colors='dodgerblue')
ax5.set_xticks(x5+0.18)
ax5.set_xticklabels(['MIMICS-def','MIMICS-T','MIMICS-TS','MIMICS-$\mathregular{TSM_b}$'],rotation='45',horizontalalignment='right',size=14)
#plt.ylim(0,0.4)
ax5.set_yticks(np.arange(0,0.8,0.2))

ax6 = ax5.twinx() 
x6=np.arange(4)+0.35
aic=np.array([4.96,4.61,4.45,4.41])
ax6.bar(x6,aic,color='pink',width=0.32) #'#ffb6b9',
ax6.set_ylabel('RMSE (g kg$\mathdefault{^{-1}}$)',color='hotpink')
#ax5.yaxis.tick_left()
ax6.yaxis.set_label_position("right")
ax6.tick_params(axis='y',colors='hotpink')
ax6.set_yticks(np.arange(4,7,1))
plt.ylim(4,6)
fig.savefig('mimics_output/fig%s_1.png'%(pyID),dpi=300)


# In[267]:


a1=np.load('MIMICS-TSMb_fba002.npy')
a2=np.load('MIMICS-BCD_fba002.npy')
a3=np.load('MIMICS-BCDV_fba002.npy')


y=[a1[1],a2[1],a3[1]]
x=[a1[0],a2[0],a3[0]]

title=['(a) MIMICS-$\mathregular{TSM_b}$','(b) MIMICS-$\mathregular{BC_{D}}$','(c) MIMICS-$\mathregular{BC_{DV}}$']
num_params=[0,1,2]
#---plot-------
fig=plt.figure(figsize=(10,4))
plt.subplots_adjust(left=0.1, bottom=0.15, right=0.95, top=0.9, wspace=0., hspace=0.)  
plt.rcParams.update({'font.size': 13})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
for i in range(3):
    
    ax1 = fig.add_subplot(1,3,i+1)
    ax1.scatter(x[i],y[i], color='dodgerblue', s=16, edgecolors='k',linewidth=0.2)
    ax1.plot(np.arange(0,60,5),np.arange(0,60,5),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left')    
    plt.xlim(0,50)
    plt.ylim(0,50)
    ax1.set_xticks(np.arange(0,50,10))
        
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    R2=get_r2_numpy(x[i], y[i])
    ndf1=len(y[i])
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105        
#----------------------
#----------------------
    # aic
    def calculate_aic(n, rmse, num_params):
        aic = n * np.log(rmse) + 2 * num_params
        return aic
    #num_params=10#param_def.shape[1]
    aic= calculate_aic(ndf1, RMSE**2, num_params[i])
    #print('AIC: %.3f' % aic)
#------------------------
    if i==0:
        ax1.set_xlabel('Observed ΔSOC (g kg$\mathdefault{^{-1}}$)',x=1.5)
        ax1.set_ylabel('Simulated ΔSOC (g kg$\mathdefault{^{-1}}$)')
    if i!=0:
        ax1.set_yticklabels([])
    fig.text(0.33+i/3.5,0.7,"1:1",size=12)
    fig.text(0.12+i/3.5,0.75,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f g kg$\mathdefault{^{-1}}$' %(R2,RMSE),size=12)
    fig.text(0.12+i/3.5,0.7,'AIC =%.2f'%(aic),size=12)
    fig.text(0.12+i/3.5,0.65,'N =%.0f'%(len(y[i])),size=12)
    xx1=np.arange(1,46,5)
    yy1=A1 *xx1 +B1
    ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
fig.savefig('fig%s_sensi.png'%(pyID),dpi=300)  


# In[269]:


a1=np.load('MIMICS-BCDV_fba_opt.npy')

y=a1[1]
x=a1[0]
ndf1=len(x)
error=y-x
RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105
print('RMSE:',RMSE)    
#output=pd.DataFrame(SOM_sim)
#output.to_excel('D:/模型/MIMICS/mimics_output/mimics_zhc_output.xlsx',sheet_name='output',na_rep='NA')
#---plot-------
fig=plt.figure(1,figsize=(4,4),facecolor='white')
plt.subplots_adjust(left=0.2, bottom=0.18, right=0.95, top=0.91, wspace=0.15, hspace=0.2)  
plt.rcParams.update({'font.size': 12})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
ax = plt.subplot(1,1,1)
#ax.errorbar(SOM_obs,SOM_sim,fmt='o', mec='blue', mfc='white',ms=3,ecolor='b',elinewidth=0.6,capthick=0.6,capsize=2,label='Geisseler et al')
ax.scatter(x,y, color='dodgerblue', s=15, edgecolors='k',linewidth=0.2)
plt.xlim(0,50)
plt.ylim(0,50)
ax.set_xlabel('Observed ΔSOC (g kg$\mathdefault{^{-1}}$) ')
ax.set_ylabel('MIMICS ΔSOC (g kg$\mathdefault{^{-1}}$)')
#ax.plot(np.arange(0,50),np.arange(0,50),ls='-', lw=0.8, c='r')
ax.plot(np.arange(0,52,2),np.arange(0,52,2), c='k', lw=0.5,linestyle='--')
#ax.set_title('(a) MIMICS-T-Sorption-$\mathregular{SM_{Yan}}$', weight='bold', loc='left')

#fig.text(0.30,0.81,"SOC",size=12)
fig.text(0.80,0.75,"1:1",size=12)
#fig.text(0.15,0.25,'Vmax,Km,D,fi,Clay,C/N,NPP',size=10)
#fig.text(0.2,0.25,'Non SM',size=10,color='r')

## -------------------fit*-------------------------
from scipy import optimize
import numpy as np
from scipy import stats
import statsmodels.api as sm
import math

def f_1(xx,A,B):
    return A*xx + B
A1,B1= optimize.curve_fit(f_1,x,y)[0]
#print(A1,B1)

## R2
def get_r2_numpy(x, y):
    slope, intercept = np.polyfit(x, y, 1)
    r_squared = 1 - (sum((y - (slope * x + intercept))**2) / ((len(y) - 1) * np.var(y, ddof=1)))
    return r_squared
R2=get_r2_numpy(x, y)
print("R2:",R2)
#----------------------
# aic
def calculate_aic(n, rmse, num_params):
    aic = n * np.log(rmse) + 2 * num_params
    return aic
num_params=4#param_def.shape[1]
aic= calculate_aic(ndf1, RMSE**2, num_params)
#print('AIC: %.3f' % aic)
    
fig.text(0.25,0.8,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.1f'%(R2,RMSE),size=12)
fig.text(0.25,0.75,'AIC =%.0f'%(aic),size=12)
fig.text(0.25,0.7,'N =%.0f'%(ndf1),size=12)


##fit
xx1=np.arange(1,50,45)
yy1=A1 *xx1 +B1
ax.plot(xx1,yy1,'black',ls='-',lw=1.)
fig.savefig('fig%s_mimics_T_sorption2_HMJ.png'%(pyID),dpi=300)


# In[86]:


a1=np.load('MIMICS-def_reverse.npy')
a2=np.load('MIMICS-T_reverse.npy')
a3=np.load('MIMICS-TS_reverse.npy')
a4=np.load('MIMICS-TSMb_reverse.npy')

b1=np.load('MIMICS-def_half.npy')
b2=np.load('MIMICS-T_half.npy')
b3=np.load('MIMICS-TS_half.npy')
b4=np.load('MIMICS-TSMb_half.npy')

c1=np.load('MIMICS-def_tillage.npy')
c2=np.load('MIMICS-T_tillage.npy')
c3=np.load('MIMICS-TS_tillage.npy')
c4=np.load('MIMICS-TSMb_tillage.npy')


y=[a1[1],a2[1],a3[1],a4[1],b1[1],b2[1],b3[1],b4[1],c1[1],c2[1],c3[1],c4[1]]
x=[a1[0],a2[0],a3[0],a4[0],b1[0],b2[0],b3[0],b4[0],c1[0],c2[0],c3[0],c4[0]]


title=['(a)','(b)','(c)','(d)','(e)','(f)','(g)','(h)','(i)','(j)','(k)','(l)','(m)','(n)','(o)','(p)']
#---plot-------
fig=plt.figure(figsize=(16,12),facecolor='white')
plt.subplots_adjust(left=0.1, bottom=0.1, right=0.95, top=0.95, wspace=0., hspace=0.)  
plt.rcParams.update({'font.size': 15})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})
for i in range(12):
    
    ax1 = fig.add_subplot(3,4,i+1)
    ax1.scatter(x[i],y[i], color='dodgerblue', s=16, edgecolors='k',linewidth=0.2)
    ax1.plot(np.arange(0,70,10),np.arange(0,70,10),ls='--', lw=0.8, c='k')
    ax1.set_title(title[i], weight='bold', loc='left',y=0.9)    
    if i==4:
        ax1.set_ylabel('Simulated SOC concentrations (g kg$\mathdefault{^{-1}}$)',y=0.5)
    if i==9:
        ax1.set_xlabel('Observed SOC concentrations (g kg$\mathdefault{^{-1}}$)',x=1)
        
#-----------rmse-------
    A1,B1= optimize.curve_fit(f_1,x[i],y[i])[0]
    R2=get_r2_numpy(x[i], y[i])
    ndf1=len(x[i])  #a1.shape[1]
    error=y[i]-x[i]
    RMSE=math.sqrt(np.sum(error**2)/ndf1)  #4.74105        
#----------------------

    if i<4:  
        plt.xlim(0,53)
        plt.ylim(0,53)
        fig.text(0.28+i/4.8,0.88,"1:1",size=15)

        fig.text(0.13+i/4.5,0.90,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f'%(R2,RMSE),size=13)
        fig.text(0.13+i/4.5,0.88,'N =%.0f'%(a1.shape[1]),size=13)
        xx1=np.arange(2,50,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i>3 and i<8:
        #ax1.set_xticks(np.arange(0,60,10))
        plt.xlim(0,53)
        plt.ylim(0,53)
        fig.text(0.28+(i-4)/4.8,0.60,"1:1",size=15)
        
        fig.text(0.13+(i-4)/4.5,0.62,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f'%(R2,RMSE),size=13)
        fig.text(0.13+(i-4)/4.5,0.60,'N =%.0f'%(b1.shape[1]),size=13)
        xx1=np.arange(2,50,5)

        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)
    if i>7 and i<12:
        plt.xlim(0,53)
        plt.ylim(0,53)  
        fig.text(0.28+(i-8)/4.8,0.32,"1:1",size=15)
        
        fig.text(0.13+(i-8)/4.5,0.34,'R$\mathdefault{^{2}}$=%.2f; RMSE=%.2f'%(R2,RMSE),size=13)
        fig.text(0.13+(i-8)/4.5,0.32,'N =%.0f'%(c1.shape[1]),size=13)
        xx1=np.arange(2,50,5)
        yy1=A1 *xx1 +B1
        ax1.plot(xx1,yy1,'black',ls='-',lw=1.)     
 
#----------------------
    if i!=0 and i!=4 and i!=8 and i!=12:   
        ax1.set_yticklabels([])
    if i<8:
        ax1.set_xticklabels([])
        
fig.savefig('fig%s_shortSOC_ge.png'%(pyID),dpi=300)      


# In[10]:


fig = plt.figure(figsize=[6,5]) 
plt.subplots_adjust() 
plt.subplots_adjust(left=0.2, right=0.95, hspace=0.3, wspace=0.25,bottom=0.2) 
plt.rcParams.update({'font.size': 15})
plt.rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
plt.rc('font',**{'family':'Arial'})

lw=0.5
ax1 = fig.add_subplot(111)
x1=np.arange(0,8,0.1)
y1=3.02*np.exp(-3.24*x1)+97.02*np.exp(-0.002*x1)
plt.plot(x1,y1,'dodgerblue',label='fit values',lw=2)


plt.xlabel('Year')
plt.ylabel('Biochar remaining in soil \n (% of added)')
plt.xlim(0,10)
plt.ylim(90,100)
#plt.legend(loc=4,frameon=False) #
#plt.title('curve_fit')
plt.show()




