#!/usr/bin/env python
# coding: utf-8

# In[38]:


# >>>> load module
#%matplotlib inline
import pandas as pd
import numpy as np
# from netCDF4 import Dataset
import matplotlib.pyplot as plt
import scipy.stats as ss
#import warnings; warnings.simplefilter('ignore')
import math


# In[39]:


#-------a_matrix------------
# This function is used to calculate the transfer ratio between each of the
# two carbon/litter pools.

def a_matrix():
    nx=7
    a_ma = np.zeros([nx,nx])
    for j in range(nx):
        a_ma[j,j]=-1.0
        
    #----default version : fixed CUE-------     
    a_ma[2,0]=0.5
    a_ma[2,6]=0.5
    a_ma[2,1]=0.25
    a_ma[3,1]=0.7
    a_ma[3,6]=0.7
    a_ma[3,0]=0.35
    a_out=a_ma
    return a_out
##?? cue is not defined


# In[40]:


#------kk_matrix-----有变动
def kk_matrix(fnpp,fmetave,tem,xav,xak,xdesorp,beta,kba,c1,c2,dt):
    nx=7
    xtv=100.0
    kk_out=np.zeros(nx)
    
    iabove=1
    ibelow=2
    imetabolic=1
    istructural=2
    
    
    tvref = math.sqrt(fnpp/xtv)
    tvref = max(0.6,min(1.3,tvref))# 0.8-1.2 based on Wieder et al., 2015
    tvr = 0.00052 *tvref * math.exp(0.3*fmetave)*dt# *x(4)
    tvk = 0.00024 *tvref * math.exp(0.1*fmetave)*dt#  *x(4)
    kk_out[2]=tvr
    kk_out[3]=tvk
    return kk_out


# In[41]:


# ------modelx---------------
def modelx(xav,xak,xdesorp,beta,kba,c1,c2,cpool,a_ma,litinpc,kk_ma,fnpp,tem,clay,bs,rho_b,moist_in,sdepth,dt,fr2p,fk2p,fr2c,fk2c,fr2a,fk2a):
    soilden=2.65 # g cm^-3 soil particle density
    nx=7
    ################ SOIL MOISTURE FUNCTION IN ORCHIDEE ############
    moist_coeff1=1.1
    moist_coeff2=2.4
    moist_coeff3=0.29
    moistcont_min=0.25
    #moistfunc_result=-moist_coeff1*moist_in*moist_in + moist_coeff2*moist_in - moist_coeff3
    moistfunc_result=moist_coeff1*moist_in*moist_in + moist_coeff2*moist_in + moist_coeff3
    control_moist=max(moistcont_min, min(1,moistfunc_result))  #ORCHIDEE 
    ############################################

    fm=0.05 #min(1.0,x(5)*exp(-x(4)*0.001*bulkd/soilden))
    fs=0.05 #min(1.0,x(5)*exp(-x(4)*0.001*bulkd/soilden))
    kor=4.0 # ZHC 6.0 --> 4.0
    kok=4.0 # ZHC 6.0 --> 4.0
    Q1=kor
    Q2=kok

    xvlit= 1.0 # 10.587669
    xvsoc= 1.0  # 23.601484
    xwlit= 1.0  # 2.266886
    xwsoc= 1.0  # 16.683772 
    xklit= 1.0  # 28.889871
    xksoc= 1.0  # 38.088382
    xjlit= 1.0  # 7.25338
    xjsoc= 1.0 # 52.29386
    #xav=x[0]
    #xak=x[1]
    #xdesorp= x[2] #*sum(cpool(5))/100% /(sum(cpool(5:7))/100)%%%% ZHC % 5.513930
    #fbs=math.exp(x[4]*bs)


    desorp = xdesorp * (1.5e-5) * math.exp(-1.5*clay)*dt*1.3 #(1-exp(-x(4)*cpool(5))) % 0.021570
    # ********** Vmaxt **********
    sv = 0.063
    av = xav*8.0e-6 
    bv = 5.47 #ZHC: xav*1.25e-8 ---> xav*8.0e-6
    xv1=10.0 
    xv2=2.0 
    xv3=10.0 # xv1=10.0; xv2=2.0; xv3=10.0;
    xw1=3.0 
    xw2=3.0 
    xw3=2.0   # xw1=3.0; xw2=3.0; xw3=2.0;
    vmax = av * math.exp(sv*tem + bv) * dt *control_moist  # Yan et al
    V1 =  xv1 * vmax*xvlit # 10 * vmax 
    V2 =  xv2 * vmax*xvlit # 2 * vmax
    V3 =  xv3 * vmax*xvsoc # 10 * vmax 
    W1 =  xw1 * vmax*xwlit # 3 * vmax 
    W2 =  xw2 * vmax*xwlit # 3 * vmax 
    W3 =  xw3 * vmax*xwsoc # 2 * vmax 

    # ********** Kmt **********
    sk =0.02
    ak = xak*10.0 
    bk = 3.19 # ZHC: ak = 0.015625 ---> ak = 10
    xk1 =8.0 
    xk2 = 2.0
    xk3 = 4.0      # xk1 =8.0; xk2 = 2.0; xk3 = 4.0;
    xj1=2.0 
    xj2=4.0 
    xj3 = 6.0         # xj1=2.0; xj2=4.0; xj3 = 6.0;
    xkclay = 1.0/math.exp(-2.0*math.sqrt(clay))   # ZHC: 3.0/exp(-2.0*sqrt(clay)) --->1.0/exp(-2.0*sqrt(clay))
    sk=0.017
    km =  ak*math.exp(sk*tem+bk)*control_moist# Yan et al
    K1 =  xklit*km/xk1        # km/8.0;
    K3 =  xksoc*km*xkclay/xk3  # km*(1-4)/4;
    J1 =  xjlit*km/xj1        # km/2.0
    J3 =  xjsoc*km*xkclay/xj3  # km/*(1-4)/6;
    #
    sk=0.027
    km = ak*math.exp(sk*tem+bk)*control_moist# Yan et al
    K2 =  xklit*km/xk2         # km/2.0
    J2 =  xjlit*km/xj2         # km/4.0

    # **** Changes in C pools
    # dmbc3=kk_ma(3)*(cpool(3)) # turnover of MBC3;
    # dmbc4=kk_ma(4)*(cpool(4)) # turnover of MBC4;
    dmbc3=kk_ma[2]*(cpool[2]**beta) # turnover of MBC3; .^x(6) % 1.140941
    dmbc4=kk_ma[3]*(cpool[3]**beta) # turnover of MBC4; .^x(6) % 1.140941
    #------- sorption function ------
    # from Wang et al., 2013, Mayes et al., 2012
    #kdes=0.001
    #kba=0.0006
    kdes=desorp
    kads=kdes*kba
    #c1=0.595 #0.297
    #c2=2.279 #3.355
    Qmax=10**(c1*np.log10(clay*100)+c2)*rho_b/1000
    #------- sorption func2.279 #tion ------
    y=np.zeros(nx)
    y[0] = litinpc[0]*(1.0-fm)-cpool[0] * (V1 * cpool[2]/(K1 + cpool[0])+W1*cpool[3]/(J1+cpool[0]))
    y[1] = litinpc[1]*(1.0-fs) -cpool[1]*(V2*cpool[2]/(K2+cpool[1])+W2*cpool[3]/(J2+cpool[1]))
    y[2] = a_ma[2,0]*V1*cpool[0]*cpool[2]/(K1+cpool[0])+ a_ma[2,1]*V2*cpool[1]*cpool[2]/(K2+cpool[1])+ a_ma[2,6]*V3*cpool[6]*cpool[2]/(K3+cpool[6])-dmbc3     
    y[3] = a_ma[3,0]*W1*cpool[0]*cpool[3]/(J1+cpool[0])+ a_ma[3,1]*W2*cpool[1]*cpool[3]/(J2+cpool[1])+ a_ma[3,6]*W3*cpool[6]*cpool[3]/(J3+cpool[6])-dmbc4
    y[4] = litinpc[0]*fm+fr2p*dmbc3+fk2p*dmbc4 + kads*(1-cpool[4]/Qmax)*cpool[6] - desorp*cpool[4]
    y[5] = litinpc[1]*fs+fr2c*dmbc3+fk2c*dmbc4- V2*cpool[5]*cpool[2]/(Q1*K2+cpool[5]) - W2*cpool[5]*cpool[3]/(Q2*J2+cpool[5])
    y[6] = fr2a*dmbc3+fk2a*dmbc4+ desorp*cpool[4]+ V2*cpool[5]*cpool[2]/(Q1*K2+cpool[5])+ W2*cpool[5]*cpool[3]/(Q2*J2+cpool[5])- V3*cpool[6]*cpool[2]/(K3+cpool[6]) - W3*cpool[6]*cpool[3]/(J3+cpool[6])-kads*(1-cpool[4]/Qmax)*cpool[6]   
    return y
   


# In[42]:


# --------- modeljacx ------------
def modeljacx(xav,xak,xdesorp,beta,kba,c1,c2,cpool,a_ma,litinpc,kk_ma,fnpp,tem,clay,bs,rho_b,moist_in,sdepth,dt,fr2p,fk2p,fr2c,fk2c,fr2a,fk2a):
    nx=7
    eps = 1.0e-4
    y0=np.zeros(nx)
    y1=np.zeros(nx)
    dydx=np.zeros([nx,nx])
    y0 = modelx(xav,xak,xdesorp,beta,kba,c1,c2,cpool,a_ma,litinpc,kk_ma,fnpp,tem,clay,bs,rho_b,moist_in,sdepth,dt,fr2p,fk2p,fr2c,fk2c,fr2a,fk2a)
    for j in range(nx):
        temp=cpool[j]
        h=eps*abs(temp)
        if h<=0.0:
            h=eps
        cpool[j]=temp+h
        h=cpool[j]-temp
        y1=modelx(xav,xak,xdesorp,beta,kba,c1,c2,cpool,a_ma,litinpc,kk_ma,fnpp,tem,clay,bs,rho_b,moist_in,sdepth,dt,fr2p,fk2p,fr2c,fk2c,fr2a,fk2a)
        cpool[j]=temp
        for ipool in range(nx):
            dydx[ipool,j]=(y1[ipool]-y0[ipool])/h
    return dydx


# In[43]:


#---------------- luccmp ---------------
def ludcmp(dydx,index):
    nx=7
    nmax=500
    tiny=1.0e-20
    d=1.0
    vv=np.zeros(nmax)
    #index=np.zeros(nx)
    dydx1=np.zeros([nx,nx])
    indx=np.zeros(nx)
    dum=np.zeros(nx)
    for i in range(nx):
        aamax=0.0
        for j in np.arange(nx):

            if (abs(dydx[i,j])>aamax):
                aamax=abs(dydx[i,j])
        if (aamax==0.0):
            print('singular matrix in ludcmp')
            dydx1[:,:]=dydx[:,:]
            break
        vv[i]=1./aamax
    for j in range(nx):
        for i in range(j):
            summ=dydx[i,j]
            for k in range(i):
                summ=summ-dydx[i,k]*dydx[k,j]
            dydx[i,j]=summ
        aamax=0.
        for i in range(j,nx):  ##1111111
            summ=dydx[i,j]
            for k in range(0,j):
                summ=summ-dydx[i,k]*dydx[k,j]
            dydx[i,j]=summ
            dum=vv[i]*abs(summ)
            if (dum>=aamax):
                imax=i
                aamax=dum
        if (j!=imax):
            for k in range(nx):
                dum=dydx[imax,k]
                dydx[imax,k]=dydx[j,k]
                dydx[j,k]=dum
            d=-d
            vv[imax]=vv[j]
        index[j]=imax
        if(dydx[j,j]==0):
            dydx[j,j]=tiny
        if(j!=nx-1):
            dum=1./dydx[j,j]
            for i in np.arange(j+1,nx):#11111111
                dydx[i,j]=dydx[i,j]*dum
        dydx1[:,:]=dydx[:,:]
        indx[:]=index[:]
    return dydx1,indx,d


# In[44]:


#--------- ludksb -----------
def ludksb(dydx1,indx,p):
    nx=7
    ii=-1
    #p=np.zeros(nx)
    for i in range(nx):
        ll=int(indx[i])
        sumerr=p[ll]
        p[ll]=p[i]
        if (ii!=-1):
            for j in np.arange(ii,i): 
                sumerr=sumerr-dydx1[i,j]*p[j]
        elif (sumerr!= 0):
            ii=i

        p[i]=sumerr
    for i in range(nx-1,-1,-1):   #111111
        sumerr=p[i]
        for j in np.arange(i+1,nx):
            sumerr=sumerr-dydx1[i,j]*p[j]
        p[i]=sumerr/dydx1[i,i]
    return p


# In[45]:



#----------------- mnewt -----------------
def mnewt(ntrial,npm,cpool,litinpc,a_ma,kk_ma,fnpp,xav,xak,xdesorp,beta,kba,c1,c2,tem,clay,bs,rho_b,moist_in,sdepth,dt,tolx,tolf,
          fr2p,fk2p,fr2c,fk2c,fr2a,fk2a):
    nx=7
    index=np.zeros(nx)
    indx=np.zeros(nx)
    p=np.zeros(nx)
    #p1=np.zeros(nx)
    iflagxy = 0
    for k in range(ntrial):
        y=modelx(xav,xak,xdesorp,beta,kba,c1,c2,cpool,a_ma,litinpc,kk_ma,fnpp,tem,clay,bs,rho_b,moist_in,sdepth,dt,fr2p,fk2p,fr2c,fk2c,fr2a,fk2a)
        dydx=modeljacx(xav,xak,xdesorp,beta,kba,c1,c2,cpool,a_ma,litinpc,kk_ma,fnpp,tem,clay,bs,rho_b,moist_in,sdepth,dt,fr2p,fk2p,fr2c,fk2c,fr2a,fk2a)
        errf=0.0
        for i in range(nx):
            dxy = max(abs(min(dydx[i,:])), abs(max(dydx[i,:])))
            if dxy==0.0:
                iflagxy=1
                break
            errf=errf+abs(y[i])

        if (errf<=tolf):
            break
        if (iflagxy==1):
            cpool[:] = 0.0
            break  
        for i in np.arange(nx):
            p[i]=-y[i]
        dydx1,indx,d=ludcmp(dydx,index)

        #---new from HZ1---
        iflagnon=1
        for i in range(nx):
            AA = np.isnan(dydx1[i,:])
            numAA = np.sum(AA) 
            if (numAA==nx):
                print('singular matrix in none value')
                iflagnon=0
                cpool[:] = -999
                break   
        if (iflagnon==0):
            break 
        p1=ludksb(dydx1,indx,p)
        errx=0.0  
        for i in range(nx):
            errx=errx+abs(p1[i])
            cpool[i]=min(max(cpool[i]+p1[i],0.0001),1000)
        if (errx<=tolx):
            break
    return cpool[:]


# In[62]:


# -------matrix_fun--------
def matrix_fun(xav,xak,xdesorp,beta,kba,c1,c2,obs):
    nx=7
    one_year=365
    one_day=86400
    emax=0.8
    cue_coef1=0.662
    cue_coef2=0.5
    m_fdecomp=296.8
    
    clay = 0.2
    partden=2.65 # soil particle density
    
    lignin_struc_cmatrix = 0.76  # lignin in matrix_nextlitter
    litter_struct_coef = 3. 
    cn_r=[30.,30., 7.4, 13.4, 12., 16., 10.] # C/N ratio for [abo_meta,bel_meta,abo_stru,bel_stru,active,slow,passive]
    cn_rr=7.4
    cn_rk=13.4
    
    
    dt=1 # time step = 1 hour
    nd=365
    ntrial=100
    #npm=len(x) # number of optimized parameters
    npm=3
    nobs=len(obs)
    # soil_weight=1. ;% kg, weight of soil
    tolx=0.0001
    tolf=0.000001
    f=0
    sd1=0
    sdlitt=0
    sdsoc=0

    ng=len(obs)
    nsamp=ng
    kk=np.zeros([ng,nx])
    fmic=np.zeros(ng)
    fphys=np.zeros(ng)
    fchem=np.zeros(ng)
    favail=np.zeros(ng)

    cpool_equlib=np.zeros([nobs,nx])
    cpool_eq=np.zeros(nx)
    CXY=np.zeros([nobs,2])
    ncxy=0
    #
    for ig in range(ng):
        #isam=obs.loc[group(ig),2]
        isam=ig
        #tem=obs.loc[isam,'MAT_WC']  # Celcius degree (℃)
        tem=obs.loc[isam,'MAT']
        #swc=obs.loc[isam,'SWC'] # 0-1
        #clay=obs.loc[isam,'CLAY_GSDE_30cm'] # 0-1
        clay=obs.loc[isam,'Clay']
        fnpp=obs.loc[isam,'NPP_MODIS']
        #mineral_n = obs.loc[isam,'MineralN(mg N/kg)']*0.001 # mineral N mg/kg --> g/kg
        sdepth=obs.loc[isam,'Sdepth_ini']# # cm
        bulkd=obs.loc[isam,'Con_BD'] # 
        rho_b=obs.loc[isam,'Con_BD']
        #socobs=obs.loc[isam,'SOC(g/kg)'] # gC/kg soil, SOC concentration
        #rho_b=obs.loc[isam,'BD']  # g/cm3
        moist_in=obs.loc[isam,'SM']  # unit:0-1
    #----------------------------------
        f_grain=0.45
        dleafc=fnpp*3/10*(1-f_grain)   # g C m^-2 yr^-1
        drootc=fnpp/5   # g C m^-2 yr^-1
        dwoodc=fnpp/2*(1-f_grain)  # g C m^-2 yr^-1
        cnleaf=25#30    # leaf C:N ratio
        cnroot=45  # root C:N ratio
        cnwood=50#30 # wood C:N ratio
        fligleaf=0.12#0.2 # leaf lignin:C
        fligroot=0.4#0.45 # root lignin:C
        fligwood=0.15#0.35 # wood lignin:C

        bs=obs.loc[isam,'BS_GSDE_30cm'] #base saturation  #mhan,暂时

        dleaf=0.1*dleafc/(24*365.0)/sdepth*dt    # mgc/cm3/day %*x(3)
        droot=0.1*drootc/(24*365.0)/sdepth*dt    # mgc/cm3/day %*x(3)
        dwood=0.1*dwoodc/(24*365.0)/sdepth*dt   #mgc/cm3/day %*x(3)
        fmetleaf=max(0.0,1.0*(0.85-0.013*fligleaf*cnleaf))
        fmetroot=max(0.0,1.0*(0.85-0.013*fligroot*cnroot))
        fmetwood=max(0.0,1.0*(0.85-0.013*fligwood*cnwood))
        fmetave = (dleaf*fmetleaf+droot*fmetroot+dwood*fmetwood)/(dleaf + droot + dwood+ 1.0e-10)
        # Fresh litter input at each time step
        litinpc=np.zeros(nx)
        litinpc[0]=fmetleaf*dleaf+fmetwood*dwood+fmetroot*droot
        litinpc[1]=(1-fmetleaf)*dleaf+(1-fmetwood)*dwood+(1.0-fmetroot)*droot
        #
        #cn_r[0]=litinpc[0]/(dleaf*fmetleaf/cnleaf+dwood*fmetwood/cnwood+droot*fmetroot/cnroot)
        #cn_r[1]=litinpc[1]/(dleaf*(1.0-fmetleaf)/cnleaf+dwood*(1.0-fmetwood)/cnwood+droot*(1.0-fmetroot)/cnroot)
        #cn_r[2]=cn_rr
        #cn_r[3]=cn_rk
        #
        #lignin_struc_cmatrix = min(0.99,(dleaf*(1.0-fmetleaf)*min(1.0,fligleaf/(1.0-fmetleaf))+dwood*(1.0-fmetwood)*min(1.0,fligwood/(1.0-fmetwood))+droot*(1.0-fmetroot)*min(1.0,fmetroot/(1.0-fmetroot)))/litinpc[2])
        #
        x6=0.432
        x7=0.098
        x8=10.56
        x9=29.78
        x10=2.61
        fr2p = min(1.0,x6*0.3 * math.exp(1.3*clay))   # !3.0 *x(5)*exp(-x(4)*0.001*bulkd/partden)
        fk2p = min(1.0,x7*0.20 * math.exp(0.8*clay))  # !3.0 *x(5)*exp(-x(4)*0.001*bulkd/partden)
        fr2c = min(1.0-fr2p, x8*0.10 * math.exp(-x10*fmetave))  # ! 9.0    to invoid a negative value of fr2a ZHC
        fk2c = min(1.0-fk2p, x9*0.30 * math.exp(-x10*fmetave))  # x(2*nsamp+5) ! 9.0    to invoid a negative value of fk2a ZHC
        fr2a = 1.00 - fr2p - fr2c
        fk2a = 1.00 - fk2p - fk2c
        cpool_ini=np.zeros(nx)
        cpool=np.zeros(nx)
        cpool_ini[0:7]=np.array([0.1,0.2,0.05,0.1,0.1,1.0,2.0])   # initial C pool
        cpool[0:7]=cpool_ini[0:7] # C pool
        #mineralN=np.zeros(nx)
        #mineralN[0,0]=mineral_n # Soil mineral N concentration (g N/ kg soil)
        #mineralN[0,1]=0 # Loss of N gas (g N/ kg soil)
        kk_ma = kk_matrix(fnpp,fmetave,tem,xav,xak,xdesorp,beta,kba,c1,c2,dt)
        #n_cue=np.zeros(nx)
        #n_cue[0]=mineral_n-cue_coef2
        #n_cue[1]=mineral_n-cue_coef2
        a_ma = a_matrix()  # Start simulation

        cpool[:]=cpool_ini[:]
        cpool_eq=mnewt(ntrial,npm,cpool,litinpc,a_ma,kk_ma,fnpp,xav,xak,xdesorp,beta,kba,c1,c2,tem,clay,bs,rho_b,moist_in,sdepth,dt,tolx,tolf,fr2p,fk2p,fr2c,fk2c,fr2a,fk2a)/bulkd
        cpool_equlib[isam,:]=cpool_eq[:]
        #sdsoc=sdsoc+(sum(cpool_eq[2:7])/(0.001*bulkd)-socobs)**2

    #return cpool_equlib
    return np.sum(cpool_equlib[:,4:7],axis=1)



