'''
Copyright 2015 by Tobias Houska
This file is part of Statistical Parameter Estimation Tool (SPOTPY).

:author: Tobias Houska

This example implements the python version of hymod into SPOTPY.
'''





import numpy as np
import pandas as pd
from spotpy.parameter import Uniform
from spotpy.objectivefunctions import rmse
from spotpy.examples.mimics_python.mimics_HMJ_newversion_TSMb_Biochar_run3_1_abc_short_ge5yr import matrix_funs
import os

class spot_setup(object):
    #xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    #xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    #xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    #beta    = Uniform(low=0.0 , high=2., optguess=1.0)
    fprim    = Uniform(low=-0.15 ,  high=0.15 , optguess=-0.002)    
    #Kq    = Uniform(low=0.1 , high=0.99, optguess=0.5592)
    #fake1 =spotpy.parameter.Uniform(low=0.1 , high=10, optguess=0.5592)
    #fake2 =spotpy.parameter.Uniform(low=0.1 , high=10, optguess=0.5592)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        #Transform [mm/day] into [l s-1], where 1.783 is the catchment area
        #self.Factor = 1.783 * 1000 * 1000 / (60 * 60 * 24)
        #self.cpool=np.array([2.85819914e-05,1.42262278e-05,1.00000000e-02,2.00000000e-02,1.00000000e-01,1.00000000e+00,2.00000000e+00])
        #self.sdepth=80 # cm
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'
        #climatefile=pd.read_csv(self.mimics_path+os.sep+'ICP_CN_MCMC.csv')
        #climatefile=pd.read_excel(self.mimics_path+os.sep+'Table_SI_211008_cess.xlsx')
        #climatefile = pd.read_excel(self.mimics_path+os.sep+'Table_SI_210501_cess_biochar.xlsx', 'TableS1', index_col=None, na_values=['NA']) #不分
        #climatefile=climatefile.groupby(by='LatLon',as_index=False).mean() #
        #climatefile = climatefile.reset_index(drop=True)

        #climatefile=pd.read_excel(self.mimics_path+os.sep+'Table_SI_210501_cess_allmergebc.xlsx')
        #climatefile=pd.read_excel(self.mimics_path+os.sep+'Table_SI_210501_cess_allbc222.xlsx')
        #climatefile=climatefile[climatefile['Age_BC']>=6]
        climatefile=pd.read_excel(self.mimics_path+os.sep+'Biochar.xlsx')
        climatefile=climatefile[climatefile['Age_BC']>=5]
        #climatefile=pd.read_excel(self.mimics_path+os.sep+'biochar_221123.xlsx')
        climatefile = climatefile.reset_index(drop=True)   

        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile['SOC']-climatefile['Con_SOC']
        #self.trueObs=climatefile['soc8yr']-climatefile['Con_SOC']
        #self.trueObs=(climatefile['SOC']*climatefile['BD']-climatefile['Con_SOC']*climatefile['Con_BD'])*climatefile['Sdepth_ini']*0.01  #kg/m2
        #self.trueObs=climatefile['Con_SOC']*climatefile['Con_BD']*climatefile['Sdepth_ini']*0.01*(np.exp(np.log(climatefile['SOC']/climatefile['Con_SOC']))-1)#RR

        

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_funs(x[0],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
