'''
Copyright 2015 by Tobias Houska
This file is part of Statistical Parameter Estimation Tool (SPOTPY).

:author: Tobias Houska

This example implements the python version of hymod into SPOTPY.
'''


import numpy as np
import pandas as pd
from spotpy.parameter import Uniform
from spotpy.objectivefunctions import rmse
from spotpy.examples.mimics_python.CV_validation.mimics_HMJ_newversion_T import matrix_fun
import os
#-------------------0------------------
class spot_setup0(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=0
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
#-------------------1------------------
class spot_setup1(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=1
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like

#-------------------2------------------
class spot_setup2(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=2
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
#-------------------3------------------
class spot_setup3(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=3
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
#-------------------4------------------
class spot_setup4(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=4
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
#-------------------5------------------
class spot_setup5(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=5
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
#-------------------6------------------
class spot_setup6(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=6
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
#-------------------7------------------
class spot_setup7(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=7
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
#-------------------8------------------
class spot_setup8(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=8
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like
#-------------------9------------------
class spot_setup9(object):
    xav     = Uniform(low=0. , high=30.0,  optguess=10.)
    xak     = Uniform(low=0. , high=20.0,  optguess=5.)
    xdesorp     = Uniform(low=0. , high=3.0,   optguess=0.5)
    beta    = Uniform(low=0.0 , high=2., optguess=1.0)
        
    def __init__(self, obj_func=None):
        #Just a way to keep this example flexible and applicable to various examples
        self.obj_func = obj_func  
        self.dt=1
        self.nx=7

        #Load Observation data from file
        #self.sdepth=[]
        #Find Path to mimics on users system
        self.owd = os.path.dirname(os.path.realpath(__file__))
        self.mimics_path = self.owd+os.sep+'mimics_python'+os.sep+'CV_validation'
        ntime=9
        climatefile=pd.read_excel(self.mimics_path+os.sep+'x_train%s.xlsx'%(ntime))
        climatefiley=pd.read_excel(self.mimics_path+os.sep+'y_train%s.xlsx'%(ntime))
        climatefile['Con_SOC']=climatefiley['Con_SOC']
        climatefile = climatefile.reset_index(drop=True)
        self.obs_IO=climatefile
        #self.trueObs=climatefile.loc[:,'SOC(g/kg)']
        self.trueObs=climatefile.loc[:,'Con_SOC']
        #self.trueObs=climatefile.loc[:,'SOC']
        #self.trueObs=climatefile.loc[:,'Con_SOC']*climatefile.loc[:,'Con_BD']*climatefile['Sdepth_ini']*0.01  #kg/m2

    def simulation(self,x):
        #Here the model is actualy startet with one paramter combination

        data=np.zeros(len(self.trueObs))
        data[data==0]=np.nan
        data=matrix_fun(x[0],x[1],x[2],x[3],self.obs_IO)

        #The first year of simulation data is ignored (warm-up)
        return data
        
    def evaluation(self):
        return self.trueObs[:]
    
    def objectivefunction(self,simulation,evaluation, params=None):
        #SPOTPY expects to get one or multiple values back, 
        #that define the performance of the model run
        if not self.obj_func:
            # This is used if not overwritten by user
            like = rmse(evaluation,simulation)
        else:
            #Way to ensure flexible spot setup class
            like = self.obj_func(evaluation,simulation)    
        return like

